@@ -80,9 +80,10 @@ class Email(object):
     self.name     = name
     self.provider = provider or self.manager.provider
     self.template = self.provider.getTemplate(self.name)
+    self._headers = idict()
     if default is None:
       default = adict(self.DEFAULTS)
-    for attr in (self.DEFAULTS.keys()):
+    for attr in self.DEFAULTS.keys():
       setattr(self, attr, copy.deepcopy(getattr(default, attr, None)))
 
     # self.attachmentTable    = None
@@ -125,6 +126,17 @@ class Email(object):
     '''
     del self.headers[key]
 
+  #----------------------------------------------------------------------------
+  # note: for some reason @property...(setter|deleter) don't work with
+  # with setattr(...) in python 2.7.3. ugh. need to use old-style.
+  def getHeaders(self):
+    return self._headers
+  def setHeaders(self, val):
+    self._headers = idict(val)
+  def delHeaders(self):
+    self._headers = idict()
+  headers = property(getHeaders, setHeaders, delHeaders)
+
   #----------------------------------------------------------------------------
   def getSettings(self):
     '''
@@ -168,8 +180,10 @@ class Email(object):
       if fmt in self.template.meta.formats:
         return util.parseXml(self.template.render(fmt, self.params))
     if fallbackToNone:
-      # todo: trap this against XML parse errors and return none
-      return util.parseXml(self.template.render(None, self.params))
+      try:
+        return util.parseXml(self.template.render(None, self.params))
+      except ET.ParseError:
+        return None
     return None
 
   #----------------------------------------------------------------------------
@@ -180,6 +194,8 @@ class Email(object):
     #       rather than being able to tag an existing node, such as:
     #         <p>This email was sent to <span email:header="To">...</span>.</p>
     xdoc = self.getTemplateXml()
+    if xdoc is None:
+      return ret
     etag = '{%s}header' % (xmlns,)
     for node in xdoc.iter():
       if node.tag == etag:
@@ -195,6 +211,8 @@ class Email(object):
     for att in self.template.meta.attachments or []:
       atts[att.name] = att
     xdoc = self.getTemplateXml()
+    if xdoc is None:
+      return atts.values()
     for node in xdoc.iter('{%s}attachment' % (xmlns,)):
       att = adict(
         name        = node.get('name'),
@@ -213,6 +231,8 @@ class Email(object):
     if 'css' in self.template.meta.formats:
       ret.append(self.template.render('css', self.params))
     xdoc = self.getTemplateXml()
+    if xdoc is None:
+      return ' '.join(ret)
     # todo: what if the node has a "src" attribute...
     for node in xdoc.findall('{%s}head/{%s}style[@type="text/css"]' % (htmlns, htmlns)):
       ret.append(node.text)
@@ -226,6 +246,8 @@ class Email(object):
     the email.
     '''
 
+    # todo: what if 'html' is not in `includeComponents`?
+
     if standalone:
       ret = self.getHtml()
       return self.inlineCidAttachments(ret)
@@ -279,9 +301,13 @@ class Email(object):
     generated if it were sent with the current settings. Use
     :meth:`send` to actually send the email.
     '''
+    # todo: what if 'text' is not in `includeComponents`?
     if 'text' in self.template.meta.formats:
       return self.template.render('text', self.params)
-    html = self.getHtml()
+    try:
+      html = self.getHtml()
+    except Exception:
+      return self.template.render(None, self.params)
     # todo: it would be interesting to be able to configure html2text to only
     #       put it footnotes for IMG tags that had non-"cid:" image references...
     text = html2text.html2text(html)
@@ -322,6 +348,8 @@ class Email(object):
     #       then more cleanly replace HTML entity characters...
     # todo: clean up this entity-replacement strategy
     xdoc = self.getTemplateXml()
+    if xdoc is None:
+      return self._cleanSubject(self.getText())
     etag = '{%s}subject' % (xmlns,)
     ret = []
     for node in xdoc.iter():
@@ -333,13 +361,6 @@ class Email(object):
       return ret
     return self._cleanSubject(self.getText())
 
-  # #----------------------------------------------------------------------------
-  # def getProviderAttachments(self):
-  #   # TODO: clean this up...
-  #   if self.attachmentTable is None:
-  #     self.attachmentTable = self.provider.getMap('attachments', {})
-  #   return self.attachmentTable
-
   #----------------------------------------------------------------------------
   def getAttachments(self):
     atts = {att.name: att for att in self.getTemplateAttachments()}
@@ -360,9 +381,9 @@ class Email(object):
     defaultHeaders = {
       'Subject':     lambda: self.getSubject(),
       }
-    for name, func in defaultHeaders.items():
+    for name, value in defaultHeaders.items():
       if name not in curheaders:
-        curheaders[name] = func()
+        curheaders[name] = value() if callable(value) else value
     if hasattr(self.manager, 'updateHeaders'):
       self.manager.updateHeaders(self, curheaders)
     return curheaders
@@ -373,10 +394,10 @@ class Email(object):
     Returns the raw SMTP data that would be sent if the email were sent
     with the current settings. Use :meth:`send` to actually send the email.
     '''
-    return self._getSmtpData(self.getOutputHeaders())
+    return self._getMimeMessage(self.getOutputHeaders()).as_string()
 
   #----------------------------------------------------------------------------
-  def _getSmtpData(self, curheaders):
+  def _getMimeMessage(self, curheaders):
 
     curheaders = idict(curheaders)
 
@@ -554,9 +575,6 @@ class Email(object):
     for k,v in curheaders.items():
       msg[util.smtpHeaderFormat(k)] = v
 
-    msg = msg.as_string()
-    if not msg.endswith('\n'):
-      msg += '\n'
     return msg
 
   #----------------------------------------------------------------------------
@@ -598,11 +616,15 @@ class Email(object):
         raise MissingHeader('email destination ("to") not specified')
     elif isinstance(recipients, basestring):
       recipients = [recipients]
-    if hasattr(self.manager, 'updateRecipients'):
-      recipients = self.manager.updateRecipients(self, recipients)
-    if hasattr(self.manager, 'updateMailfrom'):
-      mailfrom = self.manager.updateMailfrom(self, mailfrom)
-    self.manager.sender.send(mailfrom, recipients, self._getSmtpData(hdrs))
+    data = self._getMimeMessage(hdrs)
+    if self.manager.modifier:
+      mailfrom, recipients, data = self.manager.modifier.modify(
+        mailfrom, recipients, data)
+    if not isinstance(data, basestring):
+      data = data.as_string()
+      if not data.endswith('\n'):
+        data += '\n'
+    self.manager.sender.send(mailfrom, recipients, data)
 
 #------------------------------------------------------------------------------
 # end of $Id$
